import type { ExportFormat, ExportObjectFormats, FormatQueryOptions, ParameterizedNamedSQL, ParameterizedSQL, RQBJsonLogic, RuleGroupProcessor, RuleGroupTypeAny, SQLPreset } from "../../types/index.noReact";
import { defaultRuleGroupProcessorDrizzle } from "./defaultRuleGroupProcessorDrizzle";
import { defaultRuleGroupProcessorSequelize } from "./defaultRuleGroupProcessorSequelize";
/**
* A collection of option presets for {@link formatQuery}, specifically for SQL-based formats.
*
* @group Export
*/
export declare const sqlDialectPresets: Record<SQLPreset, FormatQueryOptions>;
/**
* A collection of option presets for {@link formatQuery}.
*
* @group Export
*/
export declare const formatQueryOptionPresets: Record<string, FormatQueryOptions>;
/**
* Generates a formatted (indented two spaces) JSON string from a query object.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny): string;
/**
* Generates a result based on the provided rule group processor.
*
* @group Export
*/
declare function formatQuery<TResult = unknown>(ruleGroup: RuleGroupTypeAny, options: FormatQueryOptions & {
	ruleGroupProcessor: RuleGroupProcessor<TResult>;
}): TResult;
/**
* Generates a {@link index!ParameterizedSQL ParameterizedSQL} object from a query object.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "parameterized" | (FormatQueryOptions & {
	format: "parameterized";
})): ParameterizedSQL;
/**
* Generates a {@link index!ParameterizedNamedSQL ParameterizedNamedSQL} object from a query object.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "parameterized_named" | (FormatQueryOptions & {
	format: "parameterized_named";
})): ParameterizedNamedSQL;
/**
* Generates a {@link index!RQBJsonLogic JsonLogic} object from a query object.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "jsonlogic" | (FormatQueryOptions & {
	format: "jsonlogic";
})): RQBJsonLogic;
/**
* Generates an ElasticSearch query object from an RQB query object.
*
* NOTE: Support for the ElasticSearch format is experimental.
* You may have better results exporting "sql" format then using
* [ElasticSearch SQL](https://www.elastic.co/guide/en/elasticsearch/reference/current/xpack-sql.html).
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "elasticsearch" | (FormatQueryOptions & {
	format: "elasticsearch";
})): Record<string, any>;
/**
* Generates a MongoDB query object from an RQB query object.
*
* This is equivalent to the "mongodb" format, but returns a JSON object
* instead of a string.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "mongodb_query" | (FormatQueryOptions & {
	format: "mongodb_query";
})): Record<string, any>;
/**
* Generates a JSON.stringify'd MongoDB query object from an RQB query object.
*
* This is equivalent to the "mongodb_query" format, but returns a string
* instead of a JSON object.
*
* @deprecated Use the "mongodb_query" format for greater flexibility.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "mongodb" | (FormatQueryOptions & {
	format: "mongodb";
})): string;
/**
* Generates a Prisma ORM query object from an RQB query object.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "prisma" | (FormatQueryOptions & {
	format: "prisma";
})): Record<string, any>;
/**
* Generates a Drizzle ORM query function from an RQB query object. The function can
* be assigned to the `where` property in the Drizzle relational queries API.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "drizzle" | (FormatQueryOptions & {
	format: "drizzle";
})): ReturnType<typeof defaultRuleGroupProcessorDrizzle>;
/**
* Generates a Sequelize query object from an RQB query object. The object can
* be assigned to the `where` property in the Sequelize query functions.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "sequelize" | (FormatQueryOptions & {
	format: "sequelize";
})): ReturnType<typeof defaultRuleGroupProcessorSequelize>;
/**
* Generates a JSONata query string from an RQB query object.
*
* NOTE: Either `parseNumbers: "strict-limited"` or `parseNumbers: true`
* are recommended for this format.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "jsonata" | (FormatQueryOptions & {
	format: "jsonata";
})): string;
/**
* Generates an LDAP query string from an RQB query object.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: "ldap" | (FormatQueryOptions & {
	format: "ldap";
})): string;
/**
* Generates a formatted (indented two spaces) JSON string from a query object.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: FormatQueryOptions): string;
/**
* Generates a query string in the requested format.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: Exclude<ExportFormat, ExportObjectFormats>): string;
/**
* Generates a query string in the requested format.
*
* @group Export
*/
declare function formatQuery(ruleGroup: RuleGroupTypeAny, options: FormatQueryOptions & {
	format: Exclude<ExportFormat, ExportObjectFormats>;
}): string;
export { formatQuery };
